#!/sbin/sh
dtc=$PWD/tools/dtc
dtp=$PWD/tools/dtp
bbox=$PWD/tools/busybox

file_getprop() { $bbox grep "^$2=" "$1" | $BB cut -d= -f2-; }
screen=$(file_getprop settings.txt do.refresh_rate)
cpu_offset=$(file_getprop settings.txt do.cpu_offset)
backup=$(file_getprop settings.txt do.backup)

if [ "$cpu_offset" = "0" ] && [ "$screen" = "0" ]; then
	exit 0
fi

$bbox dd if=/dev/block/bootdevice/by-name/boot of=$PWD/tools/boot.img
$PWD/tools/magiskboot unpack $PWD/tools/boot.img

file_getprop() { $bbox grep "^$2=" "$1" | $BB cut -d= -f2-; }
screen=$(file_getprop settings.txt do.refresh_rate)
cpu_offset=$(file_getprop settings.txt do.cpu_offset)
backup=$(file_getprop settings.txt do.backup)

$dtp -i kernel_dtb
if [ "$?" != "0" ]; then
	exit 1
fi

# decompile dtb
dtb_count=$(ls -lh kernel_dtb-* | wc -l)
board_id=$($bbox cat /proc/device-tree/qcom,board-id | $bbox xxd -p | $bbox xargs echo | $bbox sed 's/ //g' | $bbox sed 's/.\{8\}/&\n/g' | $bbox sed 's/^0\{6\}/0x/g' | $bbox sed 's/^0\{5\}/0x/g' | $bbox sed 's/^0\{4\}/0x/g' | $bbox sed 's/^0\{3\}/0x/g' | $bbox sed 's/^0\{2\}/0x/g' | $bbox sed 's/^0\{1\}x*/0x/g' | $bbox tr '\n' ' ' | $bbox sed 's/ *$/\n/g')
msm_id=$($bbox cat /proc/device-tree/qcom,msm-id | $bbox xxd -p | $bbox xargs echo | $bbox sed 's/ //g' | $bbox sed 's/.\{8\}/&\n/g' | $bbox sed 's/^0\{6\}/0x/g' | $bbox sed 's/^0\{5\}/0x/g' | $bbox sed 's/^0\{4\}/0x/g' | $bbox sed 's/^0\{3\}/0x/g' | $bbox sed 's/^0\{2\}/0x/g' | $bbox sed 's/^0\{1\}x*/0x/g' | $bbox tr '\n' ' ' | $bbox sed 's/ *$/\n/g')

i=0
while [ $i -lt $dtb_count ]; do
	$dtc -q -I dtb -O dts kernel_dtb-$i -o $PWD/tools/kernel_dtb_$i.dts
	dts_board_id=$($bbox cat $PWD/tools/kernel_dtb_$i.dts | $bbox grep qcom,board-id | $bbox sed -e 's/[\t]*qcom,board-id = <//g' | $bbox sed 's/>;//g')
	dts_msm_id=$($bbox cat $PWD/tools/kernel_dtb_$i.dts | $bbox grep qcom,msm-id | $bbox sed -e 's/[\t]*qcom,msm-id = <//g' | $bbox sed 's/>;//g')
	if [ "$dts_board_id" = "$board_id" ] && [ "$dts_msm_id" = "$msm_id" ]; then
		break
	fi
	$bbox rm -f $PWD/tools/kernel_dtb_$i.dts
	i=$((i + 1))
done
case $i in
$dtb_count)
	exit 1
;;
esac

old_screen=$($bbox cat $PWD/tools/kernel_dtb_$i.dts | $bbox grep qcom,mdss-dsi-panel-framerate | head -1 | cut -d '<' -f2|cut -d '>' -f1)
if [ "$backup" = "1" ]; then
	$bbox cp $PWD/tools/boot.img /sdcard/Android/boot-backup-$(printf %d $old_screen)-$(date "+%Y%m%d%H%M%S").img
fi

#Screen refresh rate
srr=qcom,mdss-dsi-panel-framerate
max_srr=qcom,mdss-dsi-max-refresh-rate
nsrr_=$(printf "0x%x" $screen)

if [ "$screen" = "0" ]; then
    ui_print "Screen refresh rate unchanged."
else
	$bbox sed -i "s/$srr = <[^)]*>/$srr = <$nsrr_>/g" $PWD/tools/kernel_dtb_$i.dts
	$bbox sed -i "s/$max_srr = <[^)]*>/$max_srr = <$nsrr_>/g" $PWD/tools/kernel_dtb_$i.dts
fi

# apply voltage offset!
if [ "$cpu_offset" = "0" ]; then
ui_print "Voltage unchanged."
else
$bbox cat $PWD/tools/kernel_dtb_$i.dts | $bbox grep qcom,cpr-open-loop-voltage-fuse-adjustment > $PWD/tools/filebuff_o
$bbox cat $PWD/tools/kernel_dtb_$i.dts | $bbox grep qcom,cpr-closed-loop-voltage-fuse-adjustment >> $PWD/tools/filebuff_o

cp $PWD/tools/filebuff_o $PWD/tools/filebuff_s
o_line=$($bbox cat $PWD/tools/filebuff_o | $bbox sed -e 's/[\t]*.*<//g' | $bbox sed 's/>;//g' | wc -l)

j=1
while [ $j -le $o_line ]; do
	line=$($bbox cat $PWD/tools/filebuff_o | $bbox awk "NR==$j")
	open_loop_voltage_=$(echo "$line" | $bbox sed -e 's/[\t]*.*<//g' | $bbox sed 's/>;//g' | $bbox sed 's/\(0x[^ ]* \)\{4\}/&\n/g')
	first_line=$(echo "$open_loop_voltage_" | $bbox sed -n '1p')
	second_line=$(echo "$open_loop_voltage_" | $bbox sed -n '2p')
	fourth_line=$(echo "$open_loop_voltage_" | $bbox sed -n '4p')

	loop_adjust=$(echo "$fourth_line" | $bbox sed 's/ $//g')
	new_v1=$(($(echo "$loop_adjust" | awk '{print $1}') + (9 * $cpu_offset / 10) * 1000))
	new_v2=$(($(echo "$loop_adjust" | awk '{print $2}') + (9 * $cpu_offset / 10) * 1000))
	new_v3=$(($(echo "$loop_adjust" | awk '{print $3}') + $cpu_offset * 1000))
	new_v4=$(($(echo "$loop_adjust" | awk '{print $4}') + $cpu_offset * 1000))
	new_v=$(printf "0x%x 0x%x 0x%x 0x%x\n" $new_v1 $new_v2 $new_v3 $new_v4 | $bbox sed 's/0xf\{8\}/0x/g')
	$bbox sed -i "s/$loop_adjust/$new_v/g" $PWD/tools/filebuff_s
	loop_adjust=$(echo "$second_line" | $bbox sed 's/ $//g')
	new_v1=$(($(echo "$loop_adjust" | awk '{print $1}') + (9 * $cpu_offset / 10) * 1000))
	new_v2=$(($(echo "$loop_adjust" | awk '{print $2}') + (9 * $cpu_offset / 10) * 1000))
	new_v3=$(($(echo "$loop_adjust" | awk '{print $3}') + $cpu_offset * 1000))
	new_v4=$(($(echo "$loop_adjust" | awk '{print $4}') + $cpu_offset * 1000))
	new_v=$(printf "0x%x 0x%x 0x%x 0x%x\n" $new_v1 $new_v2 $new_v3 $new_v4 | $bbox sed 's/0xf\{8\}/0x/g')
	$bbox sed -i "s/$loop_adjust/$new_v/g" $PWD/tools/filebuff_s
	loop_adjust=$(echo "$first_line" | $bbox sed 's/ $//g')
	new_v1=$(($(echo "$loop_adjust" | awk '{print $1}') + (9 * $cpu_offset / 10) * 1000))
	new_v2=$(($(echo "$loop_adjust" | awk '{print $2}') + (9 * $cpu_offset / 10) * 1000))
	new_v3=$(($(echo "$loop_adjust" | $bbox awk '{print $3}') + $cpu_offset * 1000))
	new_v4=$(($(echo "$loop_adjust" | $bbox awk '{print $4}') + $cpu_offset * 1000))
	new_v=$(printf "0x%x 0x%x 0x%x 0x%x\n" $new_v1 $new_v2 $new_v3 $new_v4 | $bbox sed 's/0xf\{8\}/0x/g')
	$bbox sed -i "s/$loop_adjust/$new_v/g" $PWD/tools/filebuff_s
	ori_line=$($bbox cat $PWD/tools/filebuff_o | $bbox awk "NR==$j")
	mod_line=$($bbox cat $PWD/tools/filebuff_s | $bbox awk "NR==$j")
	$bbox sed -i "s/$ori_line/$mod_line/g" $PWD/tools/kernel_dtb_$i.dts
	
	case $? in
	1)
		exit 1
	;;
	esac
	j=$((j + 1))
done
fi

# compile dts to dtb
$bbox cp $PWD/tools/kernel_dtb_$i.dts /sdcard/Android/backup.dts
$dtc -q -I dts -O dtb $PWD/tools/kernel_dtb_$i.dts -o kernel_dtb-$i

# generate new dtb
i=0
> kernel_dtb
while [ $i -lt $dtb_count ]; do
	$bbox cat kernel_dtb-$i >> kernel_dtb
	i=$((i + 1))
done
$PWD/tools/magiskboot repack $PWD/tools/boot.img
$bbox dd if=new-boot.img of=/dev/block/bootdevice/by-name/boot
$bbox rm -f kernel_dtb-*
$bbox rm -f new-boot.img